package gpsys.cos;

/**
 * The terrain robot itself is a robot that moves around on a terrain, bumping in the lamps.
 * Unfortunately it only has a limited number of energy units available.
 * These are divided into a number of movements.  
 * The terrainrobot therefore stops when these units have run
 * out. Lamps increase their energy at each turn, lose it when the robot bump in.
 * The robot can gain "released" energy from the lamp for itself by going on the 
 * Charging Station
 *
 * @version             1.1, 24 Mars 1999
 * @author      <a href="mailto:epierre@e-nef.com">Emmanuel PIERRE</a>
 *                      <address>DEA IARFA,</address>
 */
public class Robot implements java.io.Serializable {
	/**
	 * The terrain to be moveed.
	 */
	Terrain terrain;
	/**
	 * The X  corrdinate of the robot on the terrain.
	 */
	int x;
	/**
	 * The Y  corrdinate of the robot on the terrain.
	 */
	int y;
	/**
	 * The current direction of the robot, can be one of NORTH, EAST, SOUTH or
	 * WEST.
	 */
	int direction;
	/**
	 * The ration of movement operations available.
	 */
	int movementAvailable;

	/**
	 * The value of direction when the robot is pointing NORTH.
	 */
	public final static int NORTH	= 0;
	/**
	 * The value of direction when the robot is pointing EAST.
	 */
	public final static int EAST	= 1;
	/**
	 * The value of direction when the robot is pointing SOUTH.
	 */
	public final static int SOUTH	= 2;
	/**
	 * The value of direction when the robot is pointing WEST.
	 */
	public final static int WEST	= 3;

	/**
	 * The energy meaning the robot is safe (for IFDIE op)
	 */
	public final static int ROBOT_SAFE = 20*4/3;

	/**
	 * Construct a new robot to terrain the specified terrain.  The robot starts at
	 * the default position and direction. The number of movement operations available 
	 * are both initialised to 80.
	 *
	 * @param terrain	The terrain.
	 */
	public Robot(Terrain terrain) {
		this.terrain = terrain;
		reset();
	}

	/**
	 * Resets the robot state to default values (position = (3,5),
	 * direction = SOUTH, movementAvailable = 20*4.
	 */
	public void reset() {
		//x = 8;
		//y = 1;
		//direction = Robot.WEST;
		x = 3;
		y = 5;
		direction = Robot.SOUTH;
		movementAvailable  = 80;
	}

        /**
         * At the charging station.
         */
        public final boolean ifacs() {
                //terrain.at(x,y,this);
		if ((x==5)&&(y==7)) {
			return true;
		} else {
			return false;
		}
        }

        /**
         * Charging station ahead.
         */
        public final boolean ifcsa() {
		if (((direction==Robot.SOUTH)&&(y<7))
		  ||((direction==Robot.NORTH)&&(y>7))
		  ||((direction==Robot.EAST)&&(x<5))
		  ||((direction==Robot.WEST)&&(x>5))) {
			return true;
		} else {
			return false;
		}
        }

        /**
         * Lamp Greater on Left than Right.
         */
        public final boolean iflgr() {
		if (direction==Robot.SOUTH) {
			int el=0;
			int er=0;
			for (int j=y;j<10;j++) {
				for (int i=0;i<x;i++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						er+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						er+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						er+=terrain.l2;
				}
				for (int i=x+1;i<10;i++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						el+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						el+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						el+=terrain.l2;
				}
			}
			if (el>er) 
				return true;
			else	
				return false;
		} else

		if (direction==Robot.NORTH) {
			int el=0;
			int er=0;
			for (int j=0;j<=y;j++) {
				for (int i=x+1;i<10;i++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						er+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						er+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						er+=terrain.l2;
				}
				for (int i=0;i<x;i++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						el+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						el+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						el+=terrain.l2;
				}
			}
			if (el>er) 
				return true;
			else	
				return false;
		} else
	
		if (direction==Robot.WEST) {
			int el=0;
			int er=0;
			for (int i=0;i<x;i++) {
				for (int j=0;j<y;j++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						er+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						er+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						er+=terrain.l2;
				}
				for (int j=y+1;j<10;j++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						el+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						el+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						el+=terrain.l2;
				}
			}
			if (el>er) 
				return true;
			else	
				return false;
		} else

		if (direction==Robot.EAST) {
			int el=0;
			int er=0;
			for (int i=x+1;i<10;i++) {
				for (int j=y+1;j<10;j++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						er+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						er+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						er+=terrain.l2;
				}
				for (int j=0;j<y;j++) {
					if (terrain.terrain[i][j]==Terrain.L0)
						el+=terrain.l0;
					if (terrain.terrain[i][j]==Terrain.L1)
						el+=terrain.l1;
					if (terrain.terrain[i][j]==Terrain.L2)
						el+=terrain.l2;
				}
			}
			if (el>er) 
				return true;
			else	
				return false;
		} else
		
		return true;
        }

        /**
         * At the charging station and full.
         */
        public final boolean ifcharged() {
		if ((x==5)&&(y==7)&&(movementAvailable>(80*3/4))) 
			return true;
		else
			return false;
        }

        /**
         * If dying.
         */
        public final boolean ifdie() {
		if (movementAvailable<=ROBOT_SAFE) 
			return true;
		else
			return false;
        }

        /**
         * System Unsafe ! 
         */
        public final boolean ifsysunsafe() {
		if ((terrain.csnrj<=terrain.lmax)||(((terrain.l0+terrain.l1+terrain.l2)>=(terrain.lmax*3/2)))) 
			return true;
		else
			return false;
        }

        /**
         * Movement energy permitting, moves the robot forward by one square in the
         * current direction cutting the grass on that square.
         */
        public final void right() {
                if (movementAvailable == 0)
                        return;
		int xx=x;
		int yy=y;
                switch(direction) {
                        case Robot.NORTH:
				direction=Robot.EAST;
                                break;
                        case Robot.EAST:
                                direction=Robot.SOUTH;
                                break;
                        case Robot.SOUTH:
                                direction=Robot.WEST;
                                break;
                        case Robot.WEST:
                                direction=Robot.NORTH; 
                                break;
                }

                switch(direction) {
                        case Robot.NORTH:
                                y = (y == 0) ?  9 : (y - 1);
                                break;
                        case Robot.EAST:
                                x = (x + 1) % 10;
                                break;
                        case Robot.SOUTH:
                                y = (y + 1) % 10;
                                break;
                        case Robot.WEST:
                                x = (x == 0) ?  9 : (x - 1);
                                break;
                }
                movementAvailable=movementAvailable-1;

		switch (terrain.terrain[x][y]) {
                        case Terrain.L0:
				if (terrain.l0>7) {
					terrain.l0-=7;
					terrain.csnrj+=7;
				} else {
                                        terrain.csnrj+=terrain.l0;
                                        terrain.l0=0;
				}
				x=xx;
				y=yy;
				break;
                        case Terrain.L1:
				if (terrain.l1>7) {
					terrain.l1-=7;
					terrain.csnrj+=7;
				} else {
                                        terrain.csnrj+=terrain.l1;
                                        terrain.l1=0;
				}
				x=xx;
				y=yy;
				break;
                        case Terrain.L2:
				if (terrain.l2>7) {
					terrain.l2-=7;
					terrain.csnrj+=7;
				} else {
					terrain.csnrj+=terrain.l2;
					terrain.l2=0;
				}
				x=xx;
				y=yy;
				break;
                        case Terrain.R2:
				x=xx;
				y=yy;
				break;
                        case Terrain.R1:
				x=xx;
				y=yy;
				break;
		}
		terrain.at(x,y,this);
        }

        /**
         * Movement energy permitting, moves the robot forward by one square in the
         * current direction cutting the grass on that square.
         */
        public final void left() {
                if (movementAvailable == 0)
                        return;
                int xx=x;
                int yy=y;

                switch(direction) {
                        case Robot.NORTH:
				direction=Robot.WEST;
                                break;
                        case Robot.EAST:
                                direction=Robot.NORTH;
                                break;
                        case Robot.SOUTH:
                                direction=Robot.EAST;
                                break;
                        case Robot.WEST:
                                direction=Robot.SOUTH; 
                                break;
                }

                switch(direction) {
                        case Robot.NORTH:
                                y = (y == 0) ?  9 : (y - 1);
                                break;
                        case Robot.EAST:
                                x = (x + 1) % 10;
                                break;
                        case Robot.SOUTH:
                                y = (y + 1) % 10;
                                break;
                        case Robot.WEST:
                                x = (x == 0) ?  9 : (x - 1);
                                break;
                }
                movementAvailable=movementAvailable-1;

                switch (terrain.terrain[x][y]) {
                        case Terrain.L0:
                                if (terrain.l0>7) {
                                        terrain.l0-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l0;
                                        terrain.l0=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L1:
                                if (terrain.l1>7) {
                                        terrain.l1-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l1;
                                        terrain.l1=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L2:
                                if (terrain.l2>7) {
                                        terrain.l2-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l2;
                                        terrain.l2=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R2:
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R1:
                                x=xx;
                                y=yy;
                                break;
                }

		terrain.at(x,y,this);
        }

	/**
	 * Movement energy permitting, moves the robot forward by one square in the
	 * current direction.
	 */
	public final void forward() {
		if (movementAvailable == 0)
			return;
                int xx=x;
                int yy=y;

		switch(direction) {
			case Robot.NORTH:
				y = (y == 0) ?  9 : (y - 1);
				break;
			case Robot.EAST:
				x = (x + 1) % 10;
				break;
			case Robot.SOUTH:
				y = (y + 1) % 10;
				break;
			case Robot.WEST:
				x = (x == 0) ?  9 : (x - 1);
				break;
		}
		movementAvailable=movementAvailable-1;

                switch (terrain.terrain[x][y]) {
                        case Terrain.L0:
                                if (terrain.l0>7) {
                                        terrain.l0-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l0;
                                        terrain.l0=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L1:
                                if (terrain.l1>7) {
                                        terrain.l1-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l1;
                                        terrain.l1=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L2:
                                if (terrain.l2>7) {
                                        terrain.l2-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l2;
                                        terrain.l2=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R2:
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R1:
                                x=xx;
                                y=yy;
                                break;
                }

		terrain.at(x,y,this);
	}

	/**
	 * Obstacle detection.
	 */
	public final boolean ifoba() {
                int xx=x;
                int yy=y;

		switch(direction) {
			case Robot.NORTH:
				yy = (yy == 0) ?  9 : (yy - 1);
				break;
			case Robot.EAST:
				xx = (xx + 1) % 10;
				break;
			case Robot.SOUTH:
				yy = (yy + 1) % 10;
				break;
			case Robot.WEST:
				xx = (xx == 0) ?  9 : (xx - 1);
				break;
		}

                if ((terrain.terrain[xx][yy] != 0)&&(terrain.terrain[xx][yy] != Terrain.CS))
		        return true;
                else
                        return false;
	}

	/**
	 * Lamp detection.
	 */
	public final boolean ifal() {
                int xx=x;
                int yy=y;

		switch(direction) {
			case Robot.NORTH:
				yy = (yy == 0) ?  9 : (yy - 1);
				break;
			case Robot.EAST:
				xx = (xx + 1) % 10;
				break;
			case Robot.SOUTH:
				yy = (yy + 1) % 10;
				break;
			case Robot.WEST:
				xx = (xx == 0) ?  9 : (xx - 1);
				break;
		}

                if ((terrain.terrain[xx][yy] == Terrain.L0)||(terrain.terrain[xx][yy] == Terrain.L1)||(terrain.terrain[xx][yy] == Terrain.L2))
		        return true;
                else
                        return false;
	}

	/**
	 * Movement energy permitting, moves the robot back by one square in the
	 * current direction.
	 */
	public final void back() {
		if (movementAvailable == 0)
			return;
                int xx=x;
                int yy=y;

		switch(direction) {
			case Robot.SOUTH:
				y = (y == 0) ?  9 : (y - 1);
				break;
			case Robot.WEST:
				x = (x + 1) % 10;
				break;
			case Robot.NORTH:
				y = (y + 1) % 10;
				break;
			case Robot.EAST:
				x = (x == 0) ?  9 : (x - 1);
				break;
		}
		movementAvailable=movementAvailable-1;

                switch (terrain.terrain[x][y]) {
                        case Terrain.L0:
                                if (terrain.l0>7) {
                                        terrain.l0-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l0;
                                        terrain.l0=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L1:
                                if (terrain.l1>7) {
                                        terrain.l1-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l1;
                                        terrain.l1=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.L2:
                                if (terrain.l2>7) {
                                        terrain.l2-=7;
                                        terrain.csnrj+=7;
                                } else {
                                        terrain.csnrj+=terrain.l2;
                                        terrain.l2=0;
                                }
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R2:
                                x=xx;
                                y=yy;
                                break;
                        case Terrain.R1:
                                x=xx;
                                y=yy;
                                break;
                }

		terrain.at(x,y,this);
	}

	/**
	 * Movement energy permitting, moves the robot back by one square in the
	 * current direction.
	 */
	public final void halt() {
		if (movementAvailable == 0)
			return;
		terrain.at(x,y,this);
	}

}
