package gpsys.cos;

/**
 * The Vector2Mod10 type is a 2 element vector consisting of numbers in the
 * range 0 to 9.  The modulo 10 arithmetic is used to guarantee that the
 * elements stay in this range.
 *
 * @version             1.1, 24 Mars 1999
 * @author      <a href="mailto:epierre@e-nef.com">Emmanuel PIERRE</a>
 *                      <address>DEA IARFA,</address>
 */
public class Vector2Mod10 implements java.io.Serializable {
	/**
	 * The first element of the vector.
	 */
	int x;
	/**
	 * The second element of the vector.
	 */
	int y;

	/**
	 * The zero Vector2Mod10 (0,0).
	 */
	public static final Vector2Mod10 Zero = new Vector2Mod10(0, 0);

	/**
	 * Construct a new Vector2Mod10 object with the elements initialised to
	 * the specified values.  Modulo 10 arithmetic is used to ensure that the
	 * elements are in the range 0 to 9.
	 *
	 * @param x	The first element of the Vector.
	 * @param x	The second element of the Vector.
	 */
	public Vector2Mod10(int x, int y) {
		this.x = x % 10;
		this.y = y % 10;
	}
		
	/**
	 * Construct a new Vector2Mod10 object which is the sum of the specified
	 * Vector2Mod10 objects.  Modulo 10 arithmetic is used to ensure that the
	 * elements are in the range 0 to 9.
	 *
	 * @param v1	The first Vector2Mod10 object.
	 * @param v2	The second Vector2Mod10 object.
	 */
	public static final Vector2Mod10 add(Vector2Mod10 v1, Vector2Mod10 v2) {
		int xnew = (v1.x + v2.x) % 10;
		int ynew = (v1.y + v2.y) % 10;
		return new Vector2Mod10(xnew, ynew);
	}

	/**
	 * Creates a String representing this Vector2Mod10 object.
	 *
	 * @return A String representing this object.
	 */
	public String toString() {
		return "(" + x + "," + y + ")";
	}
}
